@@ -109,8 +109,8 @@ runChecker checker config verbosity check' = do
 checkDestination :: Checker ()
 checkDestination = do
     config <- checkerConfig <$> ask
-    files  <- liftIO $
-        getRecursiveContents (const False) (destinationDirectory config)
+    files  <- liftIO $ getRecursiveContents
+        (const $ return False) (destinationDirectory config)
 
     let htmls =
             [ destinationDirectory config </> file
@@ -131,9 +131,20 @@ checkFile filePath = do
     let urls = getUrls $ TS.parseTags contents
     forM_ urls $ \url -> do
         Logger.debug logger $ "Checking link " ++ url
-        if isExternal url
-            then checkExternalUrl url
-            else checkInternalUrl filePath url
+        checkUrl filePath url
+
+
+--------------------------------------------------------------------------------
+checkUrl :: FilePath -> String -> Checker ()
+checkUrl filePath url
+    | isExternal url  = checkExternalUrl url
+    | hasProtocol url = skip "Unknown protocol, skipping"
+    | otherwise       = checkInternalUrl filePath url
+  where
+    validProtoChars = ['A'..'Z'] ++ ['a'..'z'] ++ ['0'..'9'] ++ "+-."
+    hasProtocol str = case break (== ':') str of
+        (proto, ':' : _) -> all (`elem` validProtoChars) proto
+        _                -> False
 
 
 --------------------------------------------------------------------------------
@@ -141,6 +152,13 @@ ok :: String -> Checker ()
 ok _ = tell $ mempty {checkerOk = 1}
 
 
+--------------------------------------------------------------------------------
+skip :: String -> Checker ()
+skip reason = do
+    logger <- checkerLogger <$> ask
+    Logger.debug logger $ reason
+    tell $ mempty {checkerOk = 1}
+
 --------------------------------------------------------------------------------
 faulty :: String -> Checker ()
 faulty url = do
@@ -180,12 +198,14 @@ checkExternalUrl url = do
         else do
             isOk <- liftIO $ handle (failure logger) $
                 Http.withManager $ \mgr -> do
-                    request  <- Http.parseUrl url
+                    request  <- Http.parseUrl urlToCheck
                     response <- Http.http (settings request) mgr
                     let code = Http.statusCode (Http.responseStatus response)
                     return $ code >= 200 && code < 300
 
-            modify $ S.insert url
+            modify $ if schemeRelative url
+                         then S.insert urlToCheck . S.insert url
+                         else S.insert url
             if isOk then ok url else faulty url
   where
     -- Add additional request info
@@ -203,6 +223,10 @@ checkExternalUrl url = do
     failure logger (SomeException e) = case cast e of
         Just UserInterrupt -> throw UserInterrupt
         _                  -> Logger.error logger (show e) >> return False
+
+    -- Check scheme-relative links
+    schemeRelative = isPrefixOf "//"
+    urlToCheck     = if schemeRelative url then "http:" ++ url else url
 #else
 checkExternalUrl _ = return ()
 #endif
