@@ -8,9 +8,13 @@ module Hakyll.Core.Configuration
 
 
 --------------------------------------------------------------------------------
-import           Data.Default       (Default(..))
-import           Data.List          (isPrefixOf, isSuffixOf)
-import           System.FilePath    (normalise, takeFileName)
+import           Data.Default     (Default (..))
+import           Data.List        (isPrefixOf, isSuffixOf)
+import           System.Directory (canonicalizePath)
+import           System.Exit      (ExitCode)
+import           System.FilePath  (isAbsolute, normalise, takeFileName)
+import           System.IO.Error  (catchIOError)
+import           System.Process   (system)
 
 
 --------------------------------------------------------------------------------
@@ -52,6 +56,16 @@ data Configuration = Configuration
       -- > ./site deploy
       --
       deployCommand        :: String
+    , -- | Function to deploy the site from Haskell.
+      --
+      -- By default, this command executes the shell command stored in
+      -- 'deployCommand'. If you override it, 'deployCommand' will not
+      -- be used implicitely.
+      --
+      -- The 'Configuration' object is passed as a parameter to this
+      -- function.
+      --
+      deploySite           :: Configuration -> IO ExitCode
     , -- | Use an in-memory cache for items. This is faster but uses more
       -- memory.
       inMemoryCache        :: Bool
@@ -70,7 +84,8 @@ defaultConfiguration = Configuration
     , tmpDirectory         = "_cache/tmp"
     , providerDirectory    = "."
     , ignoreFile           = ignoreFile'
-    , deployCommand        = "echo 'No deploy command specified'"
+    , deployCommand        = "echo 'No deploy command specified' && exit 1"
+    , deploySite           = system . deployCommand
     , inMemoryCache        = True
     }
   where
@@ -86,11 +101,23 @@ defaultConfiguration = Configuration
 
 --------------------------------------------------------------------------------
 -- | Check if a file should be ignored
-shouldIgnoreFile :: Configuration -> FilePath -> Bool
-shouldIgnoreFile conf path =
-    destinationDirectory conf `isPrefixOf` path' ||
-    storeDirectory conf `isPrefixOf` path' ||
-    tmpDirectory conf `isPrefixOf` path' ||
-    ignoreFile conf path'
+shouldIgnoreFile :: Configuration -> FilePath -> IO Bool
+shouldIgnoreFile conf path = orM
+    [ inDir (destinationDirectory conf)
+    , inDir (storeDirectory conf)
+    , inDir (tmpDirectory conf)
+    , return (ignoreFile conf path')
+    ]
   where
-    path' = normalise path
+    path'    = normalise path
+    absolute = isAbsolute path
+
+    inDir dir
+        | absolute  = do
+            dir' <- catchIOError (canonicalizePath dir) (const $ return dir)
+            return $ dir' `isPrefixOf` path'
+        | otherwise = return $ dir `isPrefixOf` path'
+
+    orM :: [IO Bool] -> IO Bool
+    orM []       = return False
+    orM (x : xs) = x >>= \b -> if b then return True else orM xs
