@@ -9,26 +9,30 @@ module Hakyll.Core.Store
     , new
     , set
     , get
+    , isMember
     , delete
     , hash
     ) where
 
 
 --------------------------------------------------------------------------------
-import           Control.Applicative ((<$>))
-import           Control.Exception   (IOException, handle)
-import qualified Crypto.Hash.MD5     as MD5
-import           Data.Binary         (Binary, decodeFile, encodeFile)
-import qualified Data.ByteString     as B
-import qualified Data.Cache.LRU.IO   as Lru
-import           Data.List           (intercalate)
-import qualified Data.Text           as T
-import qualified Data.Text.Encoding  as T
-import           Data.Typeable       (TypeRep, Typeable, cast, typeOf)
-import           System.Directory    (createDirectoryIfMissing)
-import           System.Directory    (doesFileExist, removeFile)
-import           System.FilePath     ((</>))
-import           Text.Printf         (printf)
+import           Control.Applicative  ((<$>))
+import           Control.Exception    (IOException, handle)
+import qualified Crypto.Hash.MD5      as MD5
+import           Data.Binary          (Binary, decode, encodeFile)
+import qualified Data.ByteString      as B
+import qualified Data.ByteString.Lazy as BL
+import qualified Data.Cache.LRU.IO    as Lru
+import           Data.List            (intercalate)
+import           Data.Maybe           (isJust)
+import qualified Data.Text            as T
+import qualified Data.Text.Encoding   as T
+import           Data.Typeable        (TypeRep, Typeable, cast, typeOf)
+import           System.Directory     (createDirectoryIfMissing)
+import           System.Directory     (doesFileExist, removeFile)
+import           System.FilePath      ((</>))
+import           System.IO            (IOMode (..), hClose, openFile)
+import           Text.Printf          (printf)
 
 
 --------------------------------------------------------------------------------
@@ -98,6 +102,12 @@ cacheLookup (Store _ (Just lru)) key = do
             Nothing -> WrongType (typeOf (undefined :: a)) (typeOf x)
 
 
+--------------------------------------------------------------------------------
+cacheIsMember :: Store -> String -> IO Bool
+cacheIsMember (Store _ Nothing)    _   = return False
+cacheIsMember (Store _ (Just lru)) key = isJust <$> Lru.lookup key lru
+
+
 --------------------------------------------------------------------------------
 -- | Auxiliary: delete an item from the in-memory cache
 cacheDelete :: Store -> String -> IO ()
@@ -132,7 +142,7 @@ get store identifier = do
                 then return NotFound
                 -- Found in the filesystem
                 else do
-                    v <- decodeFile path
+                    v <- decodeClose
                     cacheInsert store key v
                     return $ Found v
         -- Found in the in-memory map (or wrong type), just return
@@ -141,6 +151,24 @@ get store identifier = do
     key  = hash identifier
     path = storeDirectory store </> key
 
+    -- 'decodeFile' from Data.Binary which closes the file ASAP
+    decodeClose = do
+        h   <- openFile path ReadMode
+        lbs <- BL.hGetContents h
+        BL.length lbs `seq` hClose h
+        return $ decode lbs
+
+
+--------------------------------------------------------------------------------
+-- | Strict function
+isMember :: Store -> [String] -> IO Bool
+isMember store identifier = do
+    inCache <- cacheIsMember store key
+    if inCache then return True else doesFileExist path
+  where
+    key  = hash identifier
+    path = storeDirectory store </> key
+
 
 --------------------------------------------------------------------------------
 -- | Delete an item
